/*
 * Copyright (c) 1997-2006 Erez Zadok
 * Copyright (c) 1997-2006 Nikolai Joukov
 * Copyright (c) 2001-2006 Stony Brook University
 * Copyright (c) 1997-2000 Columbia University
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */
/*
 *  $Id: xattr.c,v 1.12 2006/12/31 01:43:58 crpatti Exp $
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */
#include "fist.h"
#include "raif.h"

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,20)
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)

#define XATTR_BODY(func, param) \
	struct dentry *hidden_dentry = NULL; \
	int err = -EOPNOTSUPP; \
	\
	print_entry_location(); \
	hidden_dentry = dtohd_index(dentry, 0); \
	PASSERT(hidden_dentry); \
	PASSERT(hidden_dentry->d_inode); \
	PASSERT(hidden_dentry->d_inode->i_op); \
	if (hidden_dentry->d_inode->i_op->func) { \
		down(&hidden_dentry->d_inode->i_sem); \
		/* lock_kernel() already done by caller.*/ \
		err = hidden_dentry->d_inode->i_op->param; \
		/* unlock_kernel() will be done by caller.*/ \
		up(&hidden_dentry->d_inode->i_sem); \
	} \
	print_exit_status(err); \
	return err;
#else

#define XATTR_BODY(func, param) \
	struct dentry *hidden_dentry = NULL; \
	int err = -EOPNOTSUPP; \
	\
	print_entry_location(); \
	hidden_dentry = dtohd_index(dentry, 0); \
	PASSERT(hidden_dentry); \
	PASSERT(hidden_dentry->d_inode); \
	PASSERT(hidden_dentry->d_inode->i_op); \
	if (hidden_dentry->d_inode->i_op->func) { \
		mutex_lock(&hidden_dentry->d_inode->i_mutex); \
		/* lock_kernel() already done by caller.*/ \
		err = hidden_dentry->d_inode->i_op->param; \
		/* unlock_kernel() will be done by caller.*/ \
		mutex_unlock(&hidden_dentry->d_inode->i_mutex); \
	} \
	print_exit_status(err); \
	return err;
#endif

int raif_getxattr(struct dentry *dentry, const char *name, void *value, size_t size) {
	//fist_dprint(8, "getxattr: name=\"%s\", value %d bytes\n", name, size);
	XATTR_BODY(getxattr, getxattr(hidden_dentry, name, value, size));
}

int
#ifndef FIST_SETXATTR_CONSTVOID
raif_setxattr(struct dentry *dentry, const char *name, void *value, size_t size, int flags)
#else 
raif_setxattr(struct dentry *dentry, const char *name, const void *value, size_t size, int flags)
#endif /* FIST_SETXATTR_CONSTVOID */
{
	XATTR_BODY(setxattr, setxattr(hidden_dentry, name, value, size, flags));
}

int raif_removexattr(struct dentry *dentry, const char *name) {
	XATTR_BODY(removexattr, removexattr(hidden_dentry, name));
}

int raif_listxattr(struct dentry *dentry, char *list, size_t size) {
	XATTR_BODY(listxattr, listxattr(hidden_dentry, list, size));
}

# endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,20) */
